/*
 * This file is part of VLE, a framework for multi-modeling, simulation and
 * analysis of complex dynamical systems
 * http://www.vle-project.org
 *
 * Copyright (c) 2003-2012 Gauthier Quesnel <quesnel@users.sourceforge.net>
 * Copyright (c) 2003-2012 ULCO http://www.univ-littoral.fr
 * Copyright (c) 2007-2012 INRA http://www.inra.fr
 *
 * See the AUTHORS or Authors.txt file for copyright owners and contributors
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * - The name of the author may not be used to endorse or promote products
 *   derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <iomanip>      // std::setprecision
#include <iostream>
#include <vle/extension/DifferenceEquation.hpp>// Correspond a l'extension DEVS utilise ici
#include <vle/devs/DynamicsDbg.hpp>

// Raccourcis de nommage des namespaces frequement utilises
namespace ve = vle::extension;
namespace vd = vle::devs;
namespace vv = vle::value;


// Definition du namespace de la classe du modele
namespace LotkaVolterra {

class LotkaVolterraDE_euler_cyclic: public ve::DifferenceEquation::Multiple {
public:
    LotkaVolterraDE_euler_cyclic(const vd::DynamicsInit& atom, const vd::InitEventList& events) :
        ve::DifferenceEquation::Multiple(atom, events)
    {
        // Variables d'etat gerees par ce composant
        x = createVar("x");
        y = createVar("y");

        // Variables gerees par un autre composant

        // Lecture des valeurs de parametres dans les conditions du vpz
        timeStep = vv::toDouble(events.get("time-step")) ;
        // Ces parametres ont une valeur par defaut utilise si la condition n'est pas definie
        alpha = (events.exist("alpha")) ? vv::toDouble(events.get("alpha")) : 0.5;
        beta = (events.exist("beta")) ? vv::toDouble(events.get("beta")) : 0.5;
        delta = (events.exist("delta")) ? vv::toDouble(events.get("delta")) : 0.5;
        gamma = (events.exist("gamma")) ? vv::toDouble(events.get("gamma")) : 0.5;

        log = (events.exist("log")) ? vv::toBoolean(events.get("log")) : false;
    }

    virtual ~LotkaVolterraDE_euler_cyclic() {};

    virtual void compute(const vd::Time& time)
    { 
        x = x(-1) + dxdt(x(-1), y(-1)) * timeStep;
        y = y(-1) + dydt(x(), y(-1)) * timeStep;
        if (log) {
            std::cout << std::setprecision(10) << time << "b DE_euler x(-1) :" <<x(-1) << " DE_euler y(-1) :" << y(-1) << std::endl;
            std::cout << std::setprecision(10) << time << "a DE_euler x() :" <<x() << " DE_euler y() :" << y() << std::endl;
        }
    }

    virtual void initValue(const vd::Time& /* time */) 
    {
        x = 0.0;
        y = 0.0;
    }

private:
    Var x;
    Var y;

    //Parametres du modele
    double alpha;
    double beta;
    double delta;
    double gamma; 

    bool log;

    double timeStep;

    double dxdt(double X, double Y) const {
        return(X * (alpha - beta * Y));
    }

    double dydt(double X, double Y) const {
        return(Y * (-delta + gamma * X));
    }
};
}//namespaces
DECLARE_DYNAMICS_DBG(LotkaVolterra::LotkaVolterraDE_euler_cyclic)

