/*
 * This file is part of VLE, a framework for multi-modeling, simulation and
 * analysis of complex dynamical systems
 * http://www.vle-project.org
 *
 * Copyright (c) 2003-2012 Gauthier Quesnel <quesnel@users.sourceforge.net>
 * Copyright (c) 2003-2012 ULCO http://www.univ-littoral.fr
 * Copyright (c) 2007-2012 INRA http://www.inra.fr
 *
 * See the AUTHORS or Authors.txt file for copyright owners and contributors
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * - The name of the author may not be used to endorse or promote products
 *   derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#include <vle/extension/fsa/Statechart.hpp>// Correspond a l'extension DEVS utilise ici
#include <vle/devs/DynamicsDbg.hpp>

// Raccourcis de nommage des namespaces frequement utilises
namespace vd = vle::devs;
namespace vv = vle::value;
namespace ve = vle::extension;

// Definition du namespace de la classe du modele
namespace LotkaVolterra_test {

enum pStates {S1, S2};// liste des differents etat possibles de l'automate

class FSAperturb: public ve::fsa::Statechart {
public:
    FSAperturb(const vd::DynamicsInit& atom, const vd::InitEventList& events) :
        ve::fsa::Statechart(atom, events)
    {
        // Lecture des valeurs de parametres dans les conditions du vpz
        dayPerturb = (events.exist("dayPerturb")) ? events.getDouble("dayPerturb") : 7;
        varValuePerturb = (events.exist("varValuePerturb")) ? events.getDouble("varValuePerturb") : 10.0;
        varNamePerturb = (events.exist("varNamePerturb")) ? events.getString("varNamePerturb") : "x";

        //definition des etats possibles de l'automate
        states(this) << S1 << S2;

        //Definition des transition entre les etats
        // "when" defini la fonction determinant la date a laquelle se fera la transition
        // "send" defini un evenement sur un port de sortie dans le cas d'une transition
        transition(this, S1, S2)
            << when(&FSAperturb::datePerturb)
            << send(&FSAperturb::sendPerturb);
        transition(this, S2, S1);

        //etat initial de l'automate
        initialState(S1);
    }

    virtual ~FSAperturb() {}


    //when predicates
    vd::Time datePerturb(const vd::Time& /*time*/)
    { return dayPerturb; }


    //send
    void sendPerturb(const vd::Time& /*time*/, vd::ExternalEventList& output) const
    {
        vd::ExternalEvent* e = new vd::ExternalEvent("out");
        e << vd::attribute("name", varNamePerturb)
            << vd::attribute("value", varValuePerturb);
        output.push_back(e);
    }

    // Genere un evenement d'observation : calcul l'etat courant du modele a un temps donne et pour un port de sortie specifique
    virtual vv::Value* observation(const vd::ObservationEvent& /*event*/) const
    { return vv::Integer::create(currentState()); }

private:
    double dayPerturb;
    double varValuePerturb;
    std::string varNamePerturb;

};

}//namespace
DECLARE_DYNAMICS_DBG(LotkaVolterra_test::FSAperturb)

